<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" extension-element-prefixes="sw exsl func securitym local action set global"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:exsl="http://exslt.org/common"
	xmlns:func="http://exslt.org/functions" xmlns:set="http://exslt.org/sets"
	xmlns:sw="http://www.swsoft.com/xsl/cp" xmlns:local="http://www.swsoft.com/xsl/cp/local"
	xmlns:action="http://www.swsoft.com/xsl/cp/action" xmlns:units="http://www.swsoft.com/xsl/cp/units"
	xmlns:securitym="http://www.swsoft.com/xsl/cp/securitym"
	xmlns:global="http://www.swsoft.com/xsl/cp/global"
	xmlns:vza_securitym="http://www.swsoft.com/webservices/vzl/4.0.0/securitym">

	<sw:include href="/lib/core/units-core.xsl"/>
	<sw:include href="/lib/utils/action.xsl"/>

	<xsl:param name="vzcp-cred-repository" select="sw:document('credentials.xml')"/>
	<xsl:param name="vzcp-op-repository" select="sw:document('security_op.xml')"/>

	<!--
		List security tasks
		Parameters: none
		Returns: node-set with list of 'task' elements
	-->
	<func:function name="securitym:list-tasks">
		<xsl:variable name="resp" select="sw:xchg(securitym:cmd-get-task())"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym/task"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<!--
		Auxiliary template. Constructs tasks tree from VZA tasks and credentials map file
		-->
	<xsl:template name="local:construct-tasks-tree">
		<xsl:param name="task"/>
		<xsl:param name="all-tasks"/>
		<xsl:variable name="cred" select="$vzcp-cred-repository/credentials//cred[vza-task=string($task/name)]"/>
		<xsl:element name="task">
			<xsl:attribute name="order">
				<xsl:value-of
					select="$cred/numberid"/>
			</xsl:attribute>
			<xsl:copy-of select="$task/name"/>
			<xsl:element name="description">
				<xsl:value-of select="$cred/tip-text"/>
			</xsl:element>
			<xsl:for-each select="$task/task[not(name)]">
				<xsl:call-template name="local:construct-tasks-tree">
					<xsl:with-param name="task" select="$all-tasks[string(name)=current()]"/>
					<xsl:with-param name="all-tasks" select="$all-tasks"/>
				</xsl:call-template>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<!--
		Get security tasks tree
		Parameters: none
		Returns: VZA security tasks as a tree
		-->
	<func:function name="securitym:get-tasks-tree">
		<xsl:variable name="all-tasks" select="securitym:list-tasks()"/>
		<xsl:variable name="root-task-name">
			<xsl:for-each select="$all-tasks[name]">
				<xsl:variable name="tmp-name" select="string(name)"/>
				<xsl:if test="count($all-tasks//task[not(name) and string(.) = $tmp-name]) = 0">
					<xsl:value-of select="$tmp-name"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="result-tree-rtf">
			<xsl:call-template name="local:construct-tasks-tree">
				<xsl:with-param name="task" select="$all-tasks[name=$root-task-name]"/>
				<xsl:with-param name="all-tasks" select="$all-tasks"/>
			</xsl:call-template>
		</xsl:variable>
		<func:result select="exsl:node-set($result-tree-rtf)"/>
	</func:function>

	<func:function name="securitym:list-roles">
		<xsl:variable name="resp" select="sw:xchg(securitym:cmd-find-role())"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym/role"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>
	
	<!--
		List roles available for assignment.
		Parameters: none
		Returns: list of roles or 'error' element
	 -->
	<func:function name="securitym:list-assignable-roles">
		<xsl:variable name="all-roles" select="securitym:list-roles()"/>
		<xsl:variable name="wellknown-role-names" select="sw:get-builtin-roles()/roles/role[wellknown]/name"/>
		<func:result select="sw:if($all-roles[self::error],$all-roles,
			$all-roles[not(name=$wellknown-role-names)])"/>
	</func:function>

	<func:function name="securitym:find-role">
		<xsl:param name="role-id"/>
		<xsl:variable name="resp" select="sw:xchg(securitym:cmd-find-role($role-id))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym/role"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<func:function name="securitym:add-role">
		<xsl:param name="role"/>

		<xsl:variable name="resp" select="sw:xchg(securitym:cmd-add-role($role))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<func:function name="securitym:update-role">
		<xsl:param name="role-id"/>
		<xsl:param name="new-role"/>
		<xsl:variable name="req">
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<edit_role>
							<name>
								<xsl:value-of disable-output-escaping="yes" select="$role-id"/>
							</name>
							<role>
								<name>
									<xsl:value-of disable-output-escaping="yes" select="$new-role/name"/>
								</name>
								<xsl:copy-of select="$new-role/*[not(self::name)]"/>
								<xsl:for-each select="$vzcp-cred-repository/credentials//cred[@hidden='true']/vza-task">
									<task><xsl:value-of select="."/></task>
								</xsl:for-each>
							</role>
						</edit_role>
					</securitym>
				</data>
			</packet>
		</xsl:variable>

		<xsl:variable name="resp" select="sw:xchg($req)"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<func:function name="securitym:action-clone-roles">
		<xsl:param name="role-ids"/>
		<func:result>
			<xsl:call-template name="action:make-packet">
				<xsl:with-param name="path" select="'actions/core/roles/clone.xsl'"/>
				<xsl:with-param name="cookie">
					<msg>
						<sched>
							<sw:text>The roles have been scheduled for cloning.</sw:text>
						</sched>
						<error>
							<sw:text>Failed to clone the roles.</sw:text>
						</error>
					</msg>
					<operation>role.clone</operation>
					<redirect-last/>
				</xsl:with-param>
				<xsl:with-param name="input">
					<data>
						<xsl:for-each select="$role-ids">
							<role>
								<name>
									<xsl:value-of disable-output-escaping="yes" select="."/>
								</name>
							</role>
						</xsl:for-each>
					</data>
				</xsl:with-param>
			</xsl:call-template>
		</func:result>
	</func:function>

	<func:function name="securitym:action-delete-roles">
		<xsl:param name="role-ids"/>
		<func:result>
			<xsl:call-template name="action:make-packet">
				<xsl:with-param name="path" select="'actions/core/roles/delete.xsl'"/>
				<xsl:with-param name="cookie">
					<msg>
						<sched>
							<sw:text>The roles have been scheduled for deletion.</sw:text>
						</sched>
						<error>
							<sw:text>Failed to delete the roles.</sw:text>
						</error>
					</msg>
					<operation>role.delete</operation>
					<redirect-last/>
				</xsl:with-param>
				<xsl:with-param name="input">
					<data>
						<xsl:for-each select="$role-ids">
							<role>
								<name>
									<xsl:value-of disable-output-escaping="yes" select="."/>
								</name>
							</role>
						</xsl:for-each>
					</data>
				</xsl:with-param>
			</xsl:call-template>
		</func:result>
	</func:function>

	<func:function name="securitym:action-delete-role-assignments">
		<xsl:param name="aids"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<xsl:call-template name="action:make-packet">
				<xsl:with-param name="path" select="'actions/sysdir/securman/perms/delete.xsl'"/>
				<xsl:with-param name="cookie">
					<msg>
						<sched>
							<sw:text>The permissions have been scheduled for deletion.</sw:text>
						</sched>
						<error>
							<sw:text>Failed to delete the permissions.</sw:text>
						</error>
					</msg>
					<operation>permission.delete</operation>
					<redirect-last/>
				</xsl:with-param>
				<xsl:with-param name="input">
					<data>
						<scope>
							<xsl:copy-of select="$scope/*"/>
						</scope>
						<xsl:for-each select="$aids">
							<role_assignment>
								<name>
									<xsl:value-of select="."/>
								</name>
							</role_assignment>
						</xsl:for-each>
					</data>
				</xsl:with-param>
			</xsl:call-template>
		</func:result>
	</func:function>

	<func:function name="securitym:action-delete-principal-assignments">
		<xsl:param name="principal-assignments"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<xsl:call-template name="action:make-packet">
				<xsl:with-param name="path" select="'actions/sysdir/securman/perms/principal-delete.xsl'"/>
				<xsl:with-param name="cookie">
					<msg>
						<sched>
							<sw:text>The permissions have been scheduled for deletion</sw:text>
						</sched>
						<error>
							<sw:text>Failed to delete the permissions.</sw:text>
						</error>
					</msg>
					<operation>permission.delete</operation>
					<redirect-last/>
				</xsl:with-param>
				<xsl:with-param name="input">
					<data>
						<scope>
							<xsl:copy-of select="$scope/*"/>
						</scope>
						<role_assignments>
							<xsl:for-each select="set:distinct($principal-assignments/role)">
								<role_assignment>
									<role>
										<xsl:value-of disable-output-escaping="yes" select="."/>
									</role>
									<xsl:copy-of select="$principal-assignments[role=current()]/member"/>
								</role_assignment>
							</xsl:for-each>
						</role_assignments>
					</data>
				</xsl:with-param>
			</xsl:call-template>
		</func:result>
	</func:function>

	<func:function name="securitym:action-add-role-assignments">
		<xsl:param name="role-assignments"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<xsl:call-template name="action:make-packet">
				<xsl:with-param name="path" select="'actions/sysdir/securman/perms/add.xsl'"/>
				<xsl:with-param name="cookie">
					<msg>
						<sched>
							<sw:text>The permissions have been scheduled for addition.</sw:text>
						</sched>
						<error>
							<sw:text>Failed to add the permissions.</sw:text>
						</error>
					</msg>
					<operation>permission.add</operation>
					<redirect-prev/>
				</xsl:with-param>
				<xsl:with-param name="input">
					<data>
						<scope>
							<xsl:copy-of select="$scope/*"/>
						</scope>
						<role_assignments>
							<xsl:for-each select="$role-assignments">
								<xsl:copy-of select="."/>
							</xsl:for-each>
						</role_assignments>
					</data>
				</xsl:with-param>
			</xsl:call-template>
		</func:result>
	</func:function>

	<func:function name="securitym:get-role-assignments">
		<xsl:param name="role"/>
		<xsl:param name="scope" select="/.."/>
		<xsl:variable name="resp" select="sw:xchg(securitym:cmd-get-role-assignments($role,$scope))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym/role_assignment"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<func:function name="securitym:list-role-assignments">
		<xsl:param name="scope" select="/.."/>
		<xsl:variable name="resp" select="sw:xchg(securitym:cmd-list-role-assignments($scope))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym/role_assignment"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<!--
		Add role assignment in given scope
		Parameters:
			role_assignment - role assignment node-set (see securitym:role-assignment)
			scope - optional scope, global scope by default (see securitym:scope)
		Returns: 'error' node on error, 'securitym' node with status on success
	-->
	<func:function name="securitym:add-role-assignment">
		<xsl:param name="role_assignment"/>
		<xsl:param name="scope" select="/.."/>
		<xsl:variable name="resp"
			select="sw:xchg(securitym:cmd-add-role-assignment($role_assignment,$scope))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<func:function name="securitym:update-role-assignment">
		<xsl:param name="role_assignment"/>
		<xsl:param name="scope" select="/.."/>
		<xsl:variable name="resp"
			select="sw:xchg(securitym:cmd-update-role-assignments($role_assignment,$scope))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

		<xsl:choose>
			<xsl:when test="$resp-error">
				<func:result select="$resp-error"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="$resp/packet/data/securitym"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<!--
		Add role assignment for user/group in given scope
		Parameter:
			pid - user or group SID
			role-name - name of role to assign
			scope - optional scope, global scope by default (see securitym:scope)
		Returns: 'error' node on error, empty or 'securitym' node on success.
		Result node is empty if assignment for principal and role already exists.
	-->
	<func:function name="securitym:add-principal-assignment">
		<xsl:param name="pid"/>
		<xsl:param name="role-name"/>
		<xsl:param name="scope" select="/.."/>
		<xsl:variable name="member-assignments"
			select="securitym:list-role-assignments($scope)[sw:lc(role)=sw:lc($role-name)]"/>
		<xsl:choose>
			<xsl:when test="$member-assignments[self::error]">
				<func:result select="$member-assignments"/>
			</xsl:when>
			<xsl:when test="count($member-assignments/member[string(.)=$pid]) &gt; 0">
				<func:result select="/.."/>
			</xsl:when>
			<xsl:when
				test="count($member-assignments) &gt; 0">
				<xsl:variable name="upd-assignment-rtf">
					<xsl:copy-of select="$member-assignments[1]/*[local-name() != 'member']"/>
					<xsl:copy-of select="$member-assignments[1]/member[not(string(.) = $pid)]"/>
					<xsl:copy-of select="$pid"/>
				</xsl:variable>
				<func:result
					select="securitym:update-role-assignment(exsl:node-set($upd-assignment-rtf),$scope)"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="securitym:add-role-assignment(
					securitym:role-assignment($role-name,$pid)/role_assignment,
					$scope)"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<func:function name="securitym:check-operation">
		<xsl:param name="op_name"/>
		<xsl:param name="scope" select="local:id-scope('00000000-0000-0000-0000-000000000000',0)"/>
		<xsl:variable name="op_id" select="$vzcp-op-repository/operations/operation[string(@id)=$op_name]/@value"/>
		<sw:die test="not($op_id)">
			securitym:check-operation: Invalid operation name - '<xsl:value-of select="$op_name"/>' 
		</sw:die>
		<xsl:variable name="scope-ref">
			<vza_securitym:scope_ref xsi:type="vza_securitym:id_scopeType">
				<xsl:copy-of select="$scope/*"/>
			</vza_securitym:scope_ref>
		</xsl:variable>
		<func:result select="sw:access-check(number($op_id),exsl:node-set($scope-ref))"/>
	</func:function>

	<!--
		Check access for given credential in given scope
		Parameters:
			VZCP credential ID
			Scope (empty for global scope, see securitym:scope)
		Returns: boolean (true for allow, false for deny)
	-->
	<func:function name="securitym:check-access">
		<xsl:param name="cred"/>
		<xsl:param name="scope" select="/.."/>

		<xsl:choose>
			<xsl:when test="$scope/type!='' and not($scope/id !='')">
				<func:result select="false()"/>
			</xsl:when>
			<xsl:when test="not($config/vzcpcon/security/enabled='false')">
				<xsl:variable name="resp"
					select="sw:xchg(securitym:cmd-check-access(
					$vzcp-cred-repository/credentials//cred[id=$cred]/vza-task,$scope))"/>
				<xsl:variable name="resp-error"
					select="$resp/packet/data/error|$resp/packet/data/securitym/error"/>

				<xsl:choose>
					<xsl:when test="$resp-error">
						<func:result select="false()"/>
					</xsl:when>
					<xsl:otherwise>
						<func:result select="sw:if($resp/packet/data/securitym/data/value='0',true(),false())"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="true()"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<!--
		Auxiliary function, see local:access-map
	-->
	<func:function name="securitym:access-map">
		<xsl:param name="elems"/>
		<xsl:choose>
			<xsl:when test="count($elems) = 0">
				<func:result select="$elems"/>
			</xsl:when>
			<xsl:when test="not($config/vzcpcon/security/enabled='false')">
				<func:result select="local:access-map($elems)"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="fake-access-rtf">
					<xsl:for-each select="$elems">
						<xsl:copy>
							<xsl:copy-of select="@*"/>
							<xsl:copy-of select="*"/>
							<allow/>
						</xsl:copy>
					</xsl:for-each>
				</xsl:variable>
				<func:result select="exsl:node-set($fake-access-rtf)"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<!--
		Auxiliary function. Check credentails for elements having
		security attribute group - @id,@security-obj-type and @security-obj-id.
		All attributes are mandatory.
		Parameters:
			elems - node-set with plain list of elements
		Returns:
			root element with copy of 'elems' elements with 'allow' or 'deny' inside each of them
	-->
	<func:function name="local:access-map">
		<xsl:param name="elems"/>

		<!-- Calculate unique set of credentials to check -->
		<xsl:variable name="uniq-creds-rtf">
			<xsl:for-each select="set:distinct($elems/@security-obj-type)">
				<xsl:variable name="tmp-obj-type" select="string(.)"/>
				<xsl:for-each select="set:distinct($elems[@security-obj-type=$tmp-obj-type]/@security-obj-id)">
					<xsl:variable name="tmp-obj-id" select="string(.)"/>

					<!-- Handle context objects and units, which requires units:get-guid call -->
					<xsl:variable name="ctxt-obj-id">
						<xsl:choose>
							<xsl:when test="$tmp-obj-type='unit.farm'">
								<xsl:value-of
									select="units:get-guid(sw:if($tmp-obj-id='default',sw:context-object-id($tmp-obj-type),$tmp-obj-id))"/>
							</xsl:when>
							<xsl:when test="$tmp-obj-id='default'">
								<xsl:value-of select="sw:context-object-id($tmp-obj-type)"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="$tmp-obj-id"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>

					<xsl:for-each
						select="set:distinct($elems[@security-obj-type=$tmp-obj-type
							and @security-obj-id=$tmp-obj-id]/@id)">
						<uniq-cred id="{current()}"
							security-obj-type="{$tmp-obj-type}"
							security-obj-id="{$tmp-obj-id}"
							obj-id="{$ctxt-obj-id}"/>
					</xsl:for-each>
				</xsl:for-each>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="uniq-creds" select="exsl:node-set($uniq-creds-rtf)"/>

		<xsl:variable name="request-rtf">
			<packet>
				<target>securitym</target>
				<data>
					<xsl:for-each select="$uniq-creds/*[@obj-id!='']">
						<xsl:copy-of
							select="exsl:node-set(securitym:cmd-check-access(
							$vzcp-cred-repository/credentials//cred[id=string(current()/@id)]/vza-task,
							local:id-scope(@obj-id,@security-obj-type)))/packet/data/securitym"/>
					</xsl:for-each>
				</data>
			</packet>
		</xsl:variable>

		<xsl:variable name="resp" select="sw:xchg(exsl:node-set($request-rtf))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error"/>

		<xsl:variable name="uniq-access-map-rtf">
			<xsl:for-each select="$uniq-creds/*[@obj-id!='']">
				<xsl:variable name="cpos" select="position()"/>
				<xsl:copy>
					<xsl:copy-of select="@*"/>
					<xsl:choose>
						<xsl:when test="not($resp-error) and $resp/packet/data/securitym[$cpos]/data/value='0'">
							<allow/>
						</xsl:when>
						<xsl:otherwise>
							<deny/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:copy>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="uniq-access-map" select="exsl:node-set($uniq-access-map-rtf)"/>

		<xsl:variable name="resp-elems-rtf">
			<xsl:for-each select="$elems">
				<xsl:variable name="elem" select="."/>
				<xsl:copy>
					<xsl:copy-of select="@*"/>
					<xsl:copy-of select="*[name()!='allow' and name()!='deny']"/>
					<xsl:choose>
						<xsl:when test="$uniq-access-map/*[@security-obj-type=string($elem/@security-obj-type)
								and @security-obj-id=string($elem/@security-obj-id)
								and @id=string($elem/@id)]/allow">
							<allow/>
						</xsl:when>
						<xsl:otherwise>
							<deny/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:copy>
			</xsl:for-each>
		</xsl:variable>

		<func:result select="exsl:node-set($resp-elems-rtf)"/>
	</func:function>

	<!--
		Check access for given list of credentials in given scope.
		See local:check-multi-access()
	-->
	<func:function name="securitym:check-multi-access">
		<xsl:param name="creds"/>
		<xsl:param name="scope" select="/.."/>
		<xsl:choose>
			<xsl:when test="$scope/type!='' and not($scope/id != '')">
				<xsl:variable name="fake-access-rtf">
					<xsl:for-each select="$creds">
						<cred id="{.}">
							<deny/>
						</cred>
					</xsl:for-each>
				</xsl:variable>
				<func:result select="exsl:node-set($fake-access-rtf)"/>
			</xsl:when>
			<xsl:when test="not($config/vzcpcon/security/enabled='false')">
				<func:result select="local:check-multi-access($creds,$scope)"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="fake-access-rtf">
					<xsl:for-each select="$creds">
						<cred id="{.}">
							<allow/>
						</cred>
					</xsl:for-each>
				</xsl:variable>
				<func:result select="exsl:node-set($fake-access-rtf)"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<!--
		Check access for given list of credentials in given scope
		Parameters:
		Node-set with credential IDs as string values
		Scope (empty for global scope, see securitym:scope)
		Returns: node-set with access list cred(@id)/allow or cred(@id)/deny
	-->
	<func:function name="local:check-multi-access">
		<xsl:param name="creds"/>
		<xsl:param name="scope" select="/.."/>
		<xsl:variable name="request-rtf">
			<packet>
				<target>securitym</target>
				<data>
					<xsl:for-each select="$creds">
						<xsl:copy-of
							select="exsl:node-set(securitym:cmd-check-access(
							$vzcp-cred-repository/credentials//cred[id=string(current())]/vza-task,$scope))/packet/data/securitym"
						/>
					</xsl:for-each>
				</data>
			</packet>
		</xsl:variable>
		<xsl:variable name="resp" select="sw:xchg(exsl:node-set($request-rtf))"/>
		<xsl:variable name="resp-error"
			select="$resp/packet/data/error"/>

		<xsl:variable name="result-rtf">
			<xsl:choose>
				<xsl:when test="$resp-error">
					<xsl:copy-of select="$resp-error"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:for-each select="$creds">
						<xsl:variable name="cpos" select="position()"/>
						<cred id="{.}">
							<xsl:choose>
								<xsl:when test="$resp/packet/data/securitym[$cpos]/data/value='0'">
									<allow/>
								</xsl:when>
								<xsl:otherwise>
									<deny/>
									<xsl:message>MULTI CRED CHECK FAILED: <xsl:value-of select="."/> on <xsl:value-of select="securitym:scope2string($scope)"/></xsl:message>
								</xsl:otherwise>
							</xsl:choose>
						</cred>
					</xsl:for-each>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<func:result select="exsl:node-set($result-rtf)"/>
	</func:function>

	<func:function name="securitym:cmd-find-role">
		<xsl:param name="role-id" select="''"/>
		<func:result>
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<get_role>
							<xsl:if test="string($role-id) != ''">
								<name>
									<xsl:value-of disable-output-escaping="yes" select="$role-id"/>
								</name>
							</xsl:if>
						</get_role>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:cmd-add-role">
		<xsl:param name="role"/>
		<func:result>
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<add_role>
							<role>
								<name>
									<xsl:value-of disable-output-escaping="yes" select="$role/name"/>
								</name>
								<xsl:copy-of select="$role/*[not(self::name)]"/>
								<xsl:for-each select="$vzcp-cred-repository/credentials//cred[@hidden='true']/vza-task">
									<task><xsl:value-of select="."/></task>
								</xsl:for-each>
							</role>
						</add_role>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:cmd-delete-role">
		<xsl:param name="role"/>
		<func:result>
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<del_role>
							<name>
								<xsl:value-of disable-output-escaping="yes" select="$role/name"/>
							</name>
						</del_role>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:cmd-get-role-assignments">
		<xsl:param name="role"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<get_role_assignment>
							<role>
								<xsl:value-of disable-output-escaping="yes" select="$role"/>
							</role>
							<xsl:if test="$scope/id">
								<vza_securitym:scope_ref xsi:type="vza_securitym:id_scopeType">
									<id>
										<xsl:value-of select="$scope/id"/>
									</id>
									<type>
										<xsl:value-of select="$scope/type"/>
									</type>
								</vza_securitym:scope_ref>
							</xsl:if>
						</get_role_assignment>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:cmd-list-role-assignments">
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<get_role_assignment>
							<xsl:if test="$scope/id">
								<vza_securitym:scope_ref xsi:type="vza_securitym:id_scopeType">
									<id>
										<xsl:value-of select="$scope/id"/>
									</id>
									<type>
										<xsl:value-of select="$scope/type"/>
									</type>
								</vza_securitym:scope_ref>
							</xsl:if>
						</get_role_assignment>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:cmd-delete-role-assignment">
		<xsl:param name="aid"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<packet log="on">
				<target>securitym</target>
				<data>
					<securitym>
						<del_role_assignment>
							<xsl:if test="$scope/id">
								<vza_securitym:scope_ref xsi:type="vza_securitym:id_scopeType">
									<id>
										<xsl:value-of select="$scope/id"/>
									</id>
									<type>
										<xsl:value-of select="$scope/type"/>
									</type>
								</vza_securitym:scope_ref>
							</xsl:if>
							<name>
								<xsl:value-of disable-output-escaping="yes" select="$aid"/>
							</name>
						</del_role_assignment>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:cmd-add-role-assignment">
		<xsl:param name="role-assignment"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<packet log="on">
				<target>securitym</target>
				<data>
					<securitym>
						<add_role_assignment>
							<xsl:if test="$scope/id">
								<vza_securitym:scope_ref xsi:type="vza_securitym:id_scopeType">
									<id>
										<xsl:value-of select="$scope/id"/>
									</id>
									<type>
										<xsl:value-of select="$scope/type"/>
									</type>
								</vza_securitym:scope_ref>
							</xsl:if>
							<role_assignment>
								<role>
									<xsl:value-of disable-output-escaping="yes" select="$role-assignment/role"/>
								</role>
								<xsl:copy-of select="$role-assignment/member"/>
							</role_assignment>
						</add_role_assignment>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<!--
		Helper function, generates packet for role assignment (multi-)update
		Parameters:
			Node-set with role assignments
		Return value:
			Node-set with 'packet' as first child
	-->
	<func:function name="securitym:cmd-update-role-assignments">
		<xsl:param name="role-assignments"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<packet log="on">
				<target>securitym</target>
				<data>
					<xsl:for-each select="$role-assignments">
						<securitym>
							<set_role_assignment>
								<xsl:choose>
									<xsl:when test="not(name != '')">
										<role>
											<xsl:value-of disable-output-escaping="yes" select="role"/>
										</role>
									</xsl:when>
									<xsl:otherwise>
										<name>
											<xsl:value-of disable-output-escaping="yes" select="name"/>
										</name>
									</xsl:otherwise>
								</xsl:choose>
								<xsl:if test="$scope/id">
									<vza_securitym:scope_ref xsi:type="vza_securitym:id_scopeType">
										<id>
											<xsl:value-of select="$scope/id"/>
										</id>
										<type>
											<xsl:value-of select="$scope/type"/>
										</type>
									</vza_securitym:scope_ref>
								</xsl:if>
								<role_assignment>
									<role>
										<xsl:value-of disable-output-escaping="yes" select="role"/>
									</role>
									<xsl:copy-of select="member"/>
								</role_assignment>
							</set_role_assignment>
						</securitym>
					</xsl:for-each>
				</data>
			</packet>
		</func:result>
	</func:function>

	<!--
		Helper function for securitym/access_check packet
		Parameters:
			VZA task name
			Scope (empty for global scope, see securitym:scope)
		Returns: node-set with single 'packet' child
	-->
	<func:function name="securitym:cmd-check-access">
		<xsl:param name="task"/>
		<xsl:param name="scope" select="/.."/>
		<func:result>
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<access_check>
							<xsl:if test="$scope/id">
								<vza_securitym:scope_ref xsi:type="vza_securitym:id_scopeType">
									<id>
										<xsl:value-of select="$scope/id"/>
									</id>
									<type>
										<xsl:value-of select="$scope/type"/>
									</type>
								</vza_securitym:scope_ref>
							</xsl:if>
							<task>
								<xsl:value-of select="$task"/>
							</task>
						</access_check>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:role-assignment">
		<xsl:param name="role-name"/>
		<xsl:param name="member-sids"/>
		<xsl:variable name="role_assignment">
			<role_assignment>
				<role>
					<xsl:value-of select="$role-name"/>
				</role>
				<xsl:for-each select="$member-sids">
					<member>
						<xsl:value-of select="."/>
					</member>
				</xsl:for-each>
			</role_assignment>
		</xsl:variable>
		<func:result select="exsl:node-set($role_assignment)"/>
	</func:function>

	<!--
		Wrapper for securitym/get_task packet
		-->
	<func:function name="securitym:cmd-get-task">
		<xsl:param name="task"/>
		<func:result>
			<packet>
				<target>securitym</target>
				<data>
					<securitym>
						<get_task>
							<xsl:if test="$task">
								<name><xsl:value-of select="$task"/></name>
							</xsl:if>
						</get_task>
					</securitym>
				</data>
			</packet>
		</func:result>
	</func:function>

	<func:function name="securitym:scope">
		<xsl:param name="obj-id"/>
		<xsl:param name="obj-type"/>
		<xsl:variable name="scope">
			<xsl:choose>
				<!-- Generic environment -->
				<xsl:when
					test="$obj-type = 'env.core.vt' or $obj-type = 've.vz.vt' or $obj-type = 'vm.vmware.vt'
					or $obj-type = 'hn.core.vt' or $obj-type = 'hn.vz.vt' or $obj-type = 'hn.vmware.vt'">
					<xsl:copy-of select="securitym:env-scope($obj-id)"/>
				</xsl:when>
				<!-- Global -->
				<xsl:when test="$obj-type = 'vzcc.vzcp' or $obj-type = 'vzpp.vzcp' or $obj-type = 'vzcp'">
					<xsl:copy-of select="/.."/>
				</xsl:when>
				<!-- Farm unit -->
				<xsl:when test="$obj-type = 'unit.farm'">
					<xsl:copy-of select="securitym:unit-scope($obj-id)"/>
				</xsl:when>
				<!-- Cluster -->
				<xsl:when test="$obj-type = 'cluster'">
					<xsl:copy-of select="securitym:cluster-scope($obj-id)"/>
				</xsl:when>
				<xsl:otherwise>
					<sw:die>Unknown object type</sw:die>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<func:result select="exsl:node-set($scope)"/>
	</func:function>

	<!--
		Helper function. Compose scope object of given type for current context
		Parameters:
			scope-type - string, scope type
		Returns: node-set with security scope
	-->
	<func:function name="securitym:context-scope">
		<xsl:param name="scope-type"/>
		<xsl:choose>
			<xsl:when test="$scope-type = 'env.core.vt' or $scope-type = 've.vz.vt' or $scope-type = 'vm.vmware.vt'">
				<func:result select="local:id-scope(sw:ve-eid(),$scope-type)"/>
			</xsl:when>
			<xsl:when test="$scope-type = 'hn.core.vt' or $scope-type = 'hn.vz.vt' or $scope-type = 'hn.vmware.vt'">
				<func:result select="local:id-scope(sw:hn-eid(),$scope-type)"/>
			</xsl:when>
			<xsl:when test="$scope-type = 'unit.farm'">
				<func:result select="units:get-guid(sw:context-object-id($scope-type),$scope-type)"/>
			</xsl:when>
			<xsl:when test="$scope-type = 'vzcc.vzcp' or $scope-type = 'vzpp.vzcp' or $scope-type = 'vzcp'">
				<func:result select="local:id-scope('00000000-0000-0000-0000-000000000000',0)"/>
			</xsl:when>
			<xsl:when test="$scope-type = 'cluster'">
				<func:result select="local:id-scope('00000000-0000-0000-0000-000000000000',3)"/>
			</xsl:when>
			<xsl:otherwise>
				<sw:die>Unknown object type</sw:die>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>

	<func:function name="securitym:env-scope">
		<xsl:param name="eid"/>
		<func:result select="local:id-scope($eid,'1')"/>
	</func:function>

	<func:function name="securitym:unit-scope">
		<xsl:param name="unit-dn"/>
		<func:result select="local:id-scope(units:get-guid($unit-dn),'2')"/>
	</func:function>

	<func:function name="securitym:cluster-scope">
		<xsl:param name="cluster-id"/>
		<func:result select="local:id-scope($cluster-id,'3')"/>
	</func:function>

	<func:function name="local:id-scope">
		<xsl:param name="scope-id"/>
		<xsl:param name="scope-type"/>
		<xsl:variable name="scope">
			<xsl:choose>
				<!-- Generic environment -->
				<xsl:when
					test="$scope-type = 'env.core.vt' or $scope-type = 've.vz.vt' or $scope-type = 'vm.vmware.vt'
					or $scope-type = 'hn.core.vt' or $scope-type = 'hn.vz.vt' or $scope-type = 'hn.vmware.vt'">
					<id><xsl:value-of select="$scope-id"/></id>
					<type>1</type>
				</xsl:when>
				<!-- Global -->
				<xsl:when test="$scope-type = 'vzcc.vzcp' or $scope-type = 'vzpp.vzcp' or $scope-type = 'vzcp'">
				</xsl:when>
				<!-- Farm unit -->
				<xsl:when test="$scope-type = 'unit.farm'">
					<id><xsl:value-of select="$scope-id"/></id>
					<type>2</type>
				</xsl:when>
				<!-- Cluster -->
				<xsl:when test="$scope-type = 'cluster'">
					<id><xsl:value-of select="$scope-id"/></id>
					<type>3</type>
				</xsl:when>
				<xsl:otherwise>
					<id><xsl:value-of select="$scope-id"/></id>
					<type><xsl:value-of select="$scope-type"/></type>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<func:result select="exsl:node-set($scope)"/>
	</func:function>

	<!--
		Returns string representation of scope
		Parameters:
			scope
		Returns: string node
	-->
	<func:function name="securitym:scope2string">
		<xsl:param name="scope"/>
		<xsl:choose>
			<xsl:when test="$scope/type = '1'">
				<func:result select="concat('ENV #',string($scope/id),' scope')"/>
			</xsl:when>
			<xsl:when test="$scope/type = '2'">
				<func:result select="concat('Unit ',string($scope/id),' scope')"/>
			</xsl:when>
			<xsl:when test="$scope/type = '3'">
				<func:result select="concat('Cluster ',string($scope/id),' scope')"/>
			</xsl:when>
			<xsl:otherwise>
				<func:result select="'global scope'"/>
			</xsl:otherwise>
		</xsl:choose>
	</func:function>
</xsl:stylesheet>
